import { Injectable } from '@angular/core';
import { Observable, Subject } from 'rxjs';
import { map } from 'rxjs/operators';
import { HttpClient } from '@angular/common/http';
import { APP_CONFIG } from './app-config.service';
import { FormatRO, IFormat, Format } from '../models/format.model';

@Injectable()
export class FormatService {

  formatServiceUrl: string;
  limit: number;
  pageNumber: number;
  sortOptions: {
    value: string,
    order: string,
  };

  private _searchChangeSubject: Subject<any>;

  constructor(
    private _httpClient: HttpClient) {
    this.formatServiceUrl = `${APP_CONFIG.resources.url}formats/`;
    this._searchChangeSubject = new Subject<any>();
    this.limit = 10;
    this.pageNumber = 1;
  }

  getFormats(): Observable<FormatRO> {
    let query = '?';
    query += `limit=${(this.limit ? this.limit : 20)}`;
    query += `&offset=${(this.pageNumber ? (this.pageNumber - 1) * this.limit : 0)}`;
    query += `&sort_by=${this.sortOptions.value}.${this.sortOptions.order}`;

    return this._httpClient.get<IFormat[]>(this.formatServiceUrl + query, { observe: 'response' }).pipe(
      map((response) => {
        const totalCount = response.headers.get('Content-Range');
        const formats = [];
        response.body.forEach((format) => {
          formats.push(new Format(format));
        });
        return new FormatRO(formats, parseInt(totalCount, 10));
      }));
  }

  getAllFormats(): Observable<Format[]> {
    return this._httpClient.get<IFormat[]>(this.formatServiceUrl).pipe(
      map(body => body.map(format => new Format(format))),
    );
  }

  findById(id): Observable<Format> {
    return this._httpClient.get<IFormat>(this.formatServiceUrl + id).pipe(
      map((response) => {
        return new Format(response);
      }),
    );
  }

  delete(id) {
    return this._httpClient.delete(this.formatServiceUrl + id, { withCredentials: true });
  }

  replaceOrCreate(data): Observable<Format> {
    if (data.id) {
      return this._httpClient.put<IFormat>(this.formatServiceUrl + data.id, data, { withCredentials: true }).pipe(
        map((response) => {
          return new Format(response);
        }),
      );
    }
    return this._httpClient.post<IFormat>(this.formatServiceUrl, data, { withCredentials: true }).pipe(
      map((response) => {
        return new Format(response);
      }),
    );
  }

  /* PAGINATION */
  paginationChanged(limit: number, pageNumber: number) {
    this.limit = limit;
    this.pageNumber = pageNumber;
    this._searchChangeSubject.next();
  }

  reverseSortOrder(): void {
    if (this.sortOptions.order === 'asc') {
      this.sortOptions.order = 'desc';
    } else {
      this.sortOptions.order = 'asc';
    }
  }

  get searchChange$(): Observable<string> {
    return this._searchChangeSubject.asObservable();
  }
}
